import unittest
from django.db.models import Q
from savory_pie.tests.django import mock_orm


class MockUser(mock_orm.Model):
    def __repr__(self):
        return '<MockUser name={}, age={}>'.format(self.name, self.age)


class TestMockORM(unittest.TestCase):
    def setUp(self):
        self.bob1 = MockUser(pk=1, name='Bob', age=12)
        self.bob2 = MockUser(pk=2, name='Bob', age=200)
        self.jill = MockUser(pk=3, name='Jill', age=45)
        self.not_bob = MockUser(pk=4, name='Not Bob', age=54)

        self.qs = mock_orm.QuerySet(self.bob1, self.bob2, self.jill, self.not_bob)

    def test_simple_filter(self):
        self.assertEqual(
            set([self.bob1, self.bob2]),
            set(self.qs.filter(name='Bob'))
        )

    def test_and_filter(self):
        self.assertEqual(
            set([self.bob1]),
            set(self.qs.filter(name='Bob', age__lt=145))
        )

    def test_q_and_filter(self):
        q = Q(name='Bob') & Q(age__lt=145)
        self.assertEqual(
            set([self.bob1]),
            set(self.qs.filter(q))
        )

    def test_q_or_filter(self):
        q = Q(name='Bob') | Q(age__lt=50)
        self.assertEqual(
            set([self.bob1, self.bob2, self.jill]),
            set(self.qs.filter(q))
        )

    def test_q_and_or_filter(self):
        q = Q(age__lt=100) & (Q(name='Bob') | Q(name='Jill'))
        self.assertEqual(
            set([self.bob1, self.jill]),
            set(self.qs.filter(q))
        )



import pytz
import unittest
from datetime import datetime, timedelta

import django
from django.db import models

from savory_pie import fields as base_fields
from savory_pie.django import resources, fields
from savory_pie.tests.mock_context import mock_context
from savory_pie.django.validators import (
    ValidationError,
    validate,
    ResourceValidator,
    DatetimeFieldSequenceValidator,
    RequiredTogetherValidator,
    FieldValidator,
    StringFieldZipcodeValidator,
    StringFieldExactMatchValidator,
    StringFieldMaxLengthValidator,
    IntFieldMinValidator,
    IntFieldMaxValidator,
    IntFieldRangeValidator,
    DatetimeFieldMinValidator,
    DatetimeFieldMaxValidator,
)


now = datetime.now().replace(tzinfo=pytz.utc)
long_ago = now - timedelta(hours=10)
ancient = long_ago - timedelta(hours=1)
later = now + timedelta(hours=1)
too_late = later + timedelta(hours=1)
ridiculous = too_late + timedelta(hours=1)

model_save_attempted = False


class NonSavingModel(models.Model):
    def save(self, *args, **kwargs):
        global model_save_attempted
        model_save_attempted = True


class Car(NonSavingModel):
    make = models.CharField(max_length=20)
    year = models.IntegerField()
    ugly = models.BooleanField()
    mileage = models.DecimalField()

    def to_json(self):
        return {
            'make': self.make,
            'year': self.year,
            'ugly': self.ugly
        }


class BugOnWindshield(NonSavingModel):
    car = models.ForeignKey(Car, related_name='bugs')
    color = models.CharField(max_length=20)


class User(models.Model):
    name = models.CharField(max_length=20)
    age = models.IntegerField()
    before = models.DateTimeField()
    after = models.DateTimeField()
    systolic_bp = models.IntegerField()
    vehicle = models.ForeignKey(Car)
    stolen_vehicle = models.ForeignKey(Car)


class CarIsStolenValidator(ResourceValidator):

    error_message = 'The car should not be stolen.'

    def check_value(self, model):
        return False


class CarNotUglyValidator(ResourceValidator):

    error_message = 'The car should not be ugly.'

    def check_value(self, model):
        return not model['ugly']


class IntFieldPrimeValidator(FieldValidator):

    json_name = 'prime_number'

    error_message = 'This should be a prime number.'

    def __init__(self, maxprime):
        self._primes = _primes = [2, 3, 5, 7]

        def test_prime(x, _primes=_primes):
            for p in _primes:
                if p * p > x:
                    return True
                if (x % p) == 0:
                    return False
        for x in range(11, maxprime + 1, 2):
            if test_prime(x):
                _primes.append(x)
        self.populate_schema()

    def check_value(self, value):
        return value in self._primes


class BugOnWindshieldResource(resources.ModelResource):
    parent_resource_path = 'bug'
    model_class = BugOnWindshield
    fields = [
        fields.AttributeField(attribute='color', type=str),
        fields.ReverseField('car')
    ]


class BugValidator(FieldValidator):
    '''
    This is a validator on a ``RelatedManagerField``. We are validating a list of
    subresources embedded in a containing resource. Since we might need to look
    at the source_dict for the containing resource to perform a correct validation,
    it is provided to the ``find_errors`` method in the ``parent_dict`` attribute
    of the last argument, which would otherwise just be a list of source_dicts for
    the subresources. All the normal list behaviors are still available.
    '''

    json_name = 'no_green_bugs_on_new_cars'   # other color bugs are OK

    error_message = 'New cars should not have green bugs on the windshield.'

    def find_errors(self, error_dict, ctx, key, resource, field, source_dict_list):
        car_year = source_dict_list.parent_dict['year']
        green_bugs = filter(lambda bug: bug['color'] == 'green', source_dict_list)
        if car_year >= 2013 and len(green_bugs) > 0:
            self._add_error(error_dict, key, self.error_message)


class CarTestResource(resources.ModelResource):
    parent_resource_path = 'cars'
    model_class = Car

    validators = [
        CarNotUglyValidator()
    ]

    fields = [
        fields.AttributeField(
            attribute='make',
            type=str,
            validator=StringFieldExactMatchValidator(
                'Toyota', error_message='why is he not driving a Toyota?'
            )
        ),
        fields.AttributeField(
            attribute='year',
            type=int,
            validator=IntFieldMinValidator(
                2010, error_message='car is too old'
            )
        ),
        fields.RelatedManagerField(
            attribute='bugs',
            resource_class=BugOnWindshieldResource,
            validator=BugValidator()
        )
    ]


class RequiredCarTestResource(resources.ModelResource):
    parent_resource_path = 'cars'
    model_class = Car

    fields = [
        fields.AttributeField(attribute='make', type=str),
        fields.AttributeField(attribute='year', type=int),
    ]

    validators = [
        RequiredTogetherValidator(
            'make',
            'year',
            null=True,
            error_message=u'Make and year are required if either is provided.'
        ),
    ]


# ALWAYS fail validation
class StolenCarTestResource(resources.ModelResource):
    parent_resource_path = 'stolencars'
    model_class = Car

    validators = [
        CarIsStolenValidator()
    ]

    fields = [
        fields.AttributeField(attribute='make', type=str),
        fields.AttributeField(attribute='year', type=int)
    ]


class UserTestResource(resources.ModelResource):
    parent_resource_path = 'users'
    model_class = User

    validators = [
        DatetimeFieldSequenceValidator('before', 'after')
    ]

    fields = [
        fields.AttributeField(
            attribute='name',
            type=str,
            validator=StringFieldExactMatchValidator('Bob')
        ),
        fields.AttributeField(
            attribute='age',
            type=int,
            validator=(
                IntFieldMinValidator(21, error_message='too young to drink'),
                IntFieldPrimeValidator(100)
            )
        ),
        fields.AttributeField(
            attribute='before',
            type=datetime,
            validator=DatetimeFieldMinValidator(
                long_ago,
                error_message='keep it recent',
            )
        ),
        fields.AttributeField(
            attribute='after',
            type=datetime,
            validator=DatetimeFieldMaxValidator(
                too_late,
                error_message='do not be late'
            )
        ),
        fields.AttributeField(
            attribute='systolic_bp',
            type=int,
            validator=IntFieldRangeValidator(
                100,
                120,
                error_message='blood pressure out of range'
            )
        ),
        fields.SubModelResourceField('vehicle', CarTestResource),
        fields.SubModelResourceField('stolen_vehicle', StolenCarTestResource, skip_validation=True),
    ]


def create_car(make, year, ugly=False):
    model = Car()
    model.make = make
    model.year = year
    model.ugly = ugly
    return model


def validate_user_resource(name, age, start, end, systolic, car=None, stolen_car=None):
    source_dict = dict(name=name, age=str(age), before=start.isoformat(), after=end.isoformat(),
                       systolicBp=int(systolic), vehicle=(car and car.to_json()),
                       stolenVehicle=(stolen_car and stolen_car.to_json()))
    return validate(mock_context(), 'user', UserTestResource(User()), source_dict)


class ValidationTestCase(unittest.TestCase):

    maxDiff = None


class NoOpField(object):
    def handle_incoming(self, ctx, source_dict, target_obj):
        pass

    def handle_outgoing(self, ctx, source_obj, target_dict):
        pass


class OptionalValidationTestCase(ValidationTestCase):

    class OptionalResource(UserTestResource):
        validators = []

        fields = [
            NoOpField()
        ]

    def test_optional_validation(self):
        """
        Resources and Fields should not be required to have validation
        """
        model = User()
        resource = self.OptionalResource(model)
        errors = validate(mock_context(), 'user', resource, {})
        self.assertEqual({}, errors)


class SimpleValidationTestCase(ValidationTestCase):

    def test_okay(self):
        errors = validate_user_resource('Bob', 23, now, later, 120)
        self.assertEqual({}, errors)

    def test_not_recent(self):
        errors = validate_user_resource('Bob', 23, ancient, long_ago, 120)
        self.assertEqual({'user.before': ['keep it recent']}, errors)

    def test_not_late(self):
        errors = validate_user_resource('Bob', 23, now, ridiculous, 120)
        self.assertEqual({'user.after': ['do not be late']}, errors)

    def test_dates_out_of_order(self):
        errors = validate_user_resource('Bob', 23, later, now, 120)
        self.assertEqual(
            {'user':
                ['Datetimes are not in expected sequence.']},
            errors)

    def test_bogus_decimal(self):
        model = Car()
        model.make = 'Toyota'
        model.year = 2010
        model.ugly = False
        resource = CarTestResource(model)
        with self.assertRaises(ValidationError):
            resource.put(mock_context(), {
                'make': 'Toyota',
                'year': '2010',
                'ugly': 'False',
                'mileage': '123abc'
            })

    def test_wrong_name(self):
        errors = validate_user_resource('Jack', 23, now, later, 120)
        self.assertEqual(
            {'user.name':
                ['This should exactly match the expected value.']},
            errors)

    def test_too_young(self):
        errors = validate_user_resource('Bob', 19, now, later, 120)
        self.assertEqual(
            {'user.age':
                ['too young to drink']},
            errors)

    def test_prime_age(self):
        errors = validate_user_resource('Bob', 24, now, later, 120)
        self.assertEqual(
            {'user.age':
                ['This should be a prime number.']},
            errors)

    def test_hypertensive(self):
        errors = validate_user_resource('Bob', 23, now, later, 140)
        self.assertEqual(
            {'user.systolicBp':
                ['blood pressure out of range']},
            errors)

    def test_hypotensive(self):
        errors = validate_user_resource('Bob', 23, now, later, 80)
        self.assertEqual(
            {'user.systolicBp':
                ['blood pressure out of range']},
            errors)

    def test_perfect_storm(self):
        errors = validate_user_resource('Jack', 18, later, now, 140)
        self.assertEqual(
            {'user':
                ['Datetimes are not in expected sequence.'],
             'user.age':
                ['too young to drink', 'This should be a prime number.'],
             'user.name':
                ['This should exactly match the expected value.'],
             'user.systolicBp':
                ['blood pressure out of range']},
            errors)

    def test_required_together_validator(self):
        resource = RequiredCarTestResource(Car())

        good = validate(mock_context(), 'RequiredTogether', resource, {})
        self.assertEqual(good, {})

        bad = validate(mock_context(), 'RequiredTogether', resource, {'make': 'Tesla'})
        self.assertEqual(bad, {'RequiredTogether': [u'Make and year are required if either is provided.']})

        bad = validate(mock_context(), 'RequiredTogether', resource, {'year': 2012})
        self.assertEqual(bad, {'RequiredTogether': [u'Make and year are required if either is provided.']})


class SubModelValidationTestCase(ValidationTestCase):

    def test_okay(self):
        car = create_car('Toyota', 2011)
        errors = validate_user_resource('Bob', 23, now, later, 120, car)
        self.assertEqual({}, errors)

    def test_ugly(self):
        car = create_car('Toyota', 2011, ugly=True)
        errors = validate_user_resource('Bob', 23, now, later, 120, car)
        self.assertEqual({'user.vehicle': ['The car should not be ugly.']}, errors)

    def test_wrong_make(self):
        car = create_car('Honda', 2012)
        errors = validate_user_resource('Bob', 23, now, later, 120, car)
        self.assertEqual({'user.vehicle.make': ['why is he not driving a Toyota?']}, errors)

    def test_too_old(self):
        car = create_car('Toyota', 2008)
        errors = validate_user_resource('Bob', 23, now, later, 120, car)
        self.assertEqual({'user.vehicle.year': ['car is too old']}, errors)

    def test_skip_validation(self):
        stolen_car = create_car('Toyota', 2011)
        errors = validate_user_resource('Bob', 23, now, later, 120, stolen_car=stolen_car)
        self.assertEqual({}, errors)


class RelatedManagerFieldValidationTestCase(ValidationTestCase):

    def setUp(self):
        # Because of the app loading refactoring introduced in Django 1.7, this step is necessary
        # See https://docs.djangoproject.com/en/dev/releases/1.7/#app-loading-refactor
        try:
            django.setup()
        except AttributeError:
            pass

        global model_save_attempted
        self.ctx = mock_context()
        self.model = Car()
        self.resource = CarTestResource(self.model)
        self.ctx.peek.return_value = self.model
        model_save_attempted = False

    def test_old_car_with_zero_bugs(self):
        self.resource.put(self.ctx, {
            'make': 'Toyota',
            'year': 2010,
            'ugly': False,
            'bugs': []
        })
        self.assertTrue(model_save_attempted)

    def test_old_car_with_green_bugs(self):
        self.resource.put(self.ctx, {
            'make': 'Toyota',
            'year': 2010,
            'ugly': False,
            'bugs': [
                {'color': 'green'},
                {'color': 'red'},
                {'color': 'green'},
                {'color': 'yellow'},
                {'color': 'green'}
            ]
        })
        self.assertTrue(model_save_attempted)

    def test_old_car_with_non_green_bugs(self):
        self.resource.put(self.ctx, {
            'make': 'Toyota',
            'year': 2010,
            'ugly': False,
            'bugs': [
                {'color': 'red'},
                {'color': 'yellow'},
                {'color': 'blue'}
            ]
        })
        self.assertTrue(model_save_attempted)

    def test_new_car_with_zero_bugs(self):
        self.resource.put(self.ctx, {
            'make': 'Toyota',
            'year': 2013,
            'ugly': False,
            'bugs': []
        })
        self.assertTrue(model_save_attempted)

    def test_new_car_with_green_bugs(self):
        with self.assertRaises(ValidationError):
            self.resource.put(self.ctx, {
                'make': 'Toyota',
                'year': 2013,
                'ugly': False,
                'bugs': [
                    {'color': 'green'},
                    {'color': 'red'},
                    {'color': 'green'},
                    {'color': 'yellow'},
                    {'color': 'green'}
                ]
            })
        self.assertFalse(model_save_attempted)

    def test_new_car_with_non_green_bugs(self):
        self.resource.put(self.ctx, {
            'make': 'Toyota',
            'year': 2013,
            'ugly': False,
            'bugs': [
                {'color': 'red'},
                {'color': 'yellow'},
                {'color': 'blue'}
            ]
        })
        self.assertTrue(model_save_attempted)


class SchemaGetTestCase(ValidationTestCase):

    def test_validation_schema_get(self):
        resource = resources.SchemaResource(UserTestResource)
        ctx = mock_context()
        ctx.build_resource_uri = lambda resource: 'uri://users/schema/'
        result = resource.get(ctx)
        self.assertEqual(
            [{
                'text': 'Datetimes are not in expected sequence.',
                'fields': 'before,after',
                'name': 'dates_in_sequence'
            }],
            result['validators']
        )
        for field_name in result['fields']:
            field = result['fields'][field_name]
            validators = field['validators']
            self.assertTrue(type(validators) is list)
            [self.assertTrue(type(v) is dict) for v in validators]
            expected = {
                'after': [{'name': 'datetime_max',
                           'text': 'do not be late',
                           'max': too_late.isoformat()}],
                'age': [{'name': 'int_min',
                         'text': 'too young to drink',
                         'min': 21},
                        {'name': 'prime_number',
                         'text': 'This should be a prime number.'}],
                'before': [{'name': 'datetime_min',
                            'text': 'keep it recent',
                            'min': long_ago.isoformat()}],
                'name': [{'expected': 'Bob',
                          'name': 'exact_string',
                          'text': 'This should exactly match the expected value.'}],
                'systolicBp': [{'max': 120,
                                'min': 100,
                                'name': 'int_range',
                                'text': 'blood pressure out of range'}],
                'vehicle': [],
                'stolenVehicle': []
            }
            self.assertEqual(expected[field_name], validators)


class KnownValidatorsTester(ValidationTestCase):

    def test_datetime_field_sequence_validator(self):
        ctx = mock_context()
        v = DatetimeFieldSequenceValidator('a', 'b', 'c')
        badness = {'foo': ['Datetimes are not in expected sequence.']}
        for (first, second, third, expected) in (
            (now, later, too_late, {}),
            (later, now, too_late, badness),
            (now, too_late, later, badness),
            (too_late, later, now, badness)
        ):
            error_dict = {}
            v.find_errors(error_dict, ctx, 'foo', None, {'a': first.isoformat(),
                                                         'b': second.isoformat(),
                                                         'c': third.isoformat()})
            self.assertEqual(expected, error_dict)

    def test_string_field_zipcode_validator(self):
        ctx = mock_context()
        v = StringFieldZipcodeValidator()
        field = base_fields.Field()
        field._attribute = 'bar'
        field._type = str
        badness = {'foo.bar': ['This should be a zipcode.']}
        for (value, expected) in (
            ('02134', {}),
            ('01701', {}),
            ('01701-7627', {}),
            ('90210', {}),
            ('65536', {}),
            ('01234567890', badness),
            ('01234-567890', badness),
        ):
            error_dict = {}
            v.find_errors(error_dict, ctx, 'foo', None, field, value)
            self.assertEqual(expected, error_dict)

    def test_string_field_exact_match_validator(self):
        ctx = mock_context()
        v = StringFieldExactMatchValidator('01701-7627')
        field = base_fields.Field()
        field._attribute = 'bar'
        field._type = str
        badness = {'foo.bar': ['This should exactly match the expected value.']}
        for (value, expected) in (
            ('01701-7627', {}),
            ('02134', badness),
            ('01701', badness),
            ('90210', badness),
            ('65536', badness),
            ('01234567890', badness),
            ('01234-567890', badness),
        ):
            error_dict = {}
            v.find_errors(error_dict, ctx, 'foo', None, field, value)
            self.assertEqual(expected, error_dict)

    def test_string_field_max_length_validator(self):
        ctx = mock_context()
        v = StringFieldMaxLengthValidator(80)
        field = base_fields.Field()
        field._attribute = 'bar'
        field._type = str
        badness = {'foo.bar': ['This should not exceed the expected string length.']}
        for (value, expected) in (
            ('A', {}),
            (5 * 'A', {}),
            (10 * 'A', {}),
            (40 * 'A', {}),
            (79 * 'A', {}),
            (80 * 'A', {}),
            (81 * 'A', badness),
            (85 * 'A', badness),
            (100 * 'A', badness),
            (200 * 'A', badness),
            (500 * 'A', badness),
        ):
            error_dict = {}
            v.find_errors(error_dict, ctx, 'foo', None, field, value)
            self.assertEqual(expected, error_dict)

    def test_int_field_min_validator(self):
        ctx = mock_context()
        v = IntFieldMinValidator(11)
        field = base_fields.Field()
        field._attribute = 'bar'
        field._type = int
        badness = {'foo.bar': ['This value should be greater than or equal to the minimum.']}
        for (value, expected) in (
            ('0', badness),
            ('5', badness),
            ('11', {}),
            ('23', {}),
            ('90210', {})
        ):
            error_dict = {}
            v.find_errors(error_dict, ctx, 'foo', None, field, value)
            self.assertEqual(expected, error_dict)

    def test_int_field_max_validator(self):
        ctx = mock_context()
        v = IntFieldMaxValidator(11)
        field = base_fields.Field()
        field._attribute = 'bar'
        field._type = int
        badness = {'foo.bar': ['This value should be less than or equal to the maximum.']}
        for (value, expected) in (
            ('0', {}),
            ('5', {}),
            ('11', {}),
            ('23', badness),
            ('90210', badness)
        ):
            error_dict = {}
            v.find_errors(error_dict, ctx, 'foo', None, field, value)
            self.assertEqual(expected, error_dict)

    def test_int_field_range_validator(self):
        ctx = mock_context()
        v = IntFieldRangeValidator(4, 11)
        field = base_fields.Field()
        field._attribute = 'bar'
        field._type = int
        badness = {'foo.bar': ['This value should be within the allowed integer range.']}
        for (value, expected) in (
            ('0', badness),
            ('3', badness),
            ('4', {}),
            ('5', {}),
            ('10', {}),
            ('11', {}),
            ('12', badness),
            ('23', badness),
            ('90210', badness)
        ):
            error_dict = {}
            v.find_errors(error_dict, ctx, 'foo', None, field, value)
            self.assertEqual(expected, error_dict)

    def test_datetime_field_min_validator(self):
        ctx = mock_context()
        v = DatetimeFieldMinValidator(now)
        field = base_fields.Field()
        field._attribute = 'bar'
        field._type = datetime
        badness = {'foo.bar': ['This value should be no earlier than the minimum datetime.']}
        for (value, expected) in (
            (ancient, badness),
            (long_ago, badness),
            (now, {}),
            (later, {}),
            (too_late, {})
        ):
            error_dict = {}
            v.find_errors(error_dict, ctx, 'foo', None, field, value.isoformat())
            self.assertEqual(expected, error_dict)

    def test_datetime_field_max_validator(self):
        ctx = mock_context()
        v = DatetimeFieldMaxValidator(now)
        field = base_fields.Field()
        field._attribute = 'bar'
        field._type = datetime
        badness = {'foo.bar': ['This value should be no later than the maximum datetime.']}
        for (value, expected) in (
            (ancient, {}),
            (long_ago, {}),
            (now, {}),
            (later, badness),
            (too_late, badness)
        ):
            error_dict = {}
            v.find_errors(error_dict, ctx, 'foo', None, field, value.isoformat())
            self.assertEqual(expected, error_dict)



from datetime import datetime
from savory_pie import formatters
import pytz

json_formatter = formatters.JSONFormatter()
date_str = json_formatter.to_api_value(datetime, datetime.now(pytz.UTC))

#TODO add filtering and sort order
user_resource_schema = {
    'allowedDetailHttpMethods': ['put', 'delete', 'get'],
    'allowedListHttpMethods': ['put', 'delete', 'get'],
    'defaultFormat': 'application/json',
    'defaultLimit': 0,
    'filtering': {},
    'ordering': [],
    'resourceUri': 'uri://user/schema/',
    'fields': {
        'dateJoined': {
            'nullable': False,
            'default': date_str,
            'readonly': False,
            'helpText': u'',
            'blank': False,
            'unique': False,
            'type': 'datetime',
            'validators': []
        },
    }
}



import unittest
from mock import Mock, patch

from savory_pie import helpers
from savory_pie.errors import MethodNotAllowedError, PreConditionError


class ResourceHelperTestCase(unittest.TestCase):
    def test_get_not_allowed(self):
        with self.assertRaises(MethodNotAllowedError):
            resource = Mock(name='resource', allowed_methods=['POST'])
            helpers.process_get_request(Mock(name='ctx'), resource, {})

    @patch('savory_pie.helpers._ParamsImpl')
    def test_get(self, ParamsImplClass):
        ParamsImplClass.return_value = 'getparams'
        resource = Mock(name='resource', allowed_methods=['GET'])
        ctx = Mock(name='ctx')
        helpers.process_get_request(ctx, resource, {})
        resource.get.assert_called_with(ctx, 'getparams')

    def test_put_not_allowed(self):
        with self.assertRaises(MethodNotAllowedError):
            resource = Mock(name='resource', allowed_methods=['POST'])
            helpers.process_put_request(Mock(name='ctx'), resource, {})

    @patch('savory_pie.helpers.EmptyParams')
    def test_put_precondition(self, EmptyParamsClass):
        EmptyParamsClass.return_value = 'params'
        with self.assertRaises(PreConditionError):
            resource = Mock(name='resource', allowed_methods=['PUT'])
            resource.get.return_value = {}
            ctx = Mock(name='ctx')
            helpers.process_put_request(ctx, resource, {'data': 'data'}, expected_hash='123')
        resource.put.assert_called_with(ctx, {'data': 'data'})
        resource.get.assert_called_with(ctx, 'params')

    def test_put(self):
        resource = Mock(name='resource', allowed_methods=['PUT'])
        resource.get.return_value = {}
        resource.put.return_value = 'some value'
        ctx = Mock(name='ctx')
        result = helpers.process_put_request(ctx, resource, {'data': 'data'})
        resource.put.assert_called_with(ctx, {'data': 'data'})
        self.assertEqual(result, 'some value')

    def test_post_not_allowed(self):
        with self.assertRaises(MethodNotAllowedError):
            resource = Mock(name='resource', allowed_methods=['GET'])
            helpers.process_post_request(Mock(name='ctx'), resource, {})

    @patch('savory_pie.helpers._ParamsImpl')
    def test_post(self, ParamsImplClass):
        ParamsImplClass.return_value = 'getparams'
        resource = Mock(name='resource', allowed_methods=['POST'])
        ctx = Mock(name='ctx')
        helpers.process_post_request(ctx, resource, {'data': 'data'})
        resource.post.assert_called_with(ctx, {'data': 'data'})

    def test_delete_not_allowed(self):
        with self.assertRaises(MethodNotAllowedError):
            resource = Mock(name='resource', allowed_methods=['GET'])
            helpers.process_delete_request(Mock(name='ctx'), resource)

    @patch('savory_pie.helpers._ParamsImpl')
    def test_delete(self, ParamsImplClass):
        ParamsImplClass.return_value = 'getparams'
        resource = Mock(name='resource', allowed_methods=['DELETE'])
        ctx = Mock(name='ctx')
        helpers.process_delete_request(ctx, resource)
        resource.delete.assert_called_with(ctx)



import mock
import unittest
from savory_pie import utils
from savory_pie.utils import ParamsDict


class ToListTest(unittest.TestCase):

    def test_to_list_splits_on_comma(self):
        ids_param = '1,2,3,4,5'
        self.assertEqual(utils.to_list(ids_param), ['1', '2', '3', '4', '5'])

    def test_to_list_bad_data(self):
        march_15_timestamp = 12345
        self.assertEqual(utils.to_list(march_15_timestamp), 12345)

    def test_not_list_split(self):
        item = mock.Mock()
        item.split.return_value = 'Something'
        self.assertEqual(utils.to_datetime(item), item)


class ToDateTimeTest(unittest.TestCase):

    def test_to_datetime(self):
        march_15_timestamp = 1363320000000
        march_15_datetime = utils.to_datetime(march_15_timestamp)

        self.assertEqual(march_15_datetime.year, 2013)
        self.assertEqual(march_15_datetime.month, 3)
        self.assertEqual(march_15_datetime.day, 15)

    def test_to_datetime_bad_date(self):
        march_15_timestamp = 'something'
        self.assertEqual(utils.to_datetime(march_15_timestamp), 'something')


class ParamsDictTestCase(unittest.TestCase):

    def test_parameters_contains(self):
        params = ParamsDict({'key1': 'value1'})
        self.assertTrue('key1' in params)

    def test_parameters_get(self):
        params = ParamsDict({'key1': 'value1'})
        self.assertEqual(params.get('key1', 'other'), 'value1')

    def test_parameters_get_default(self):
        params = ParamsDict({'key1': 'value1'})
        self.assertEqual(params.get('some', 'other'), 'other')

    def test_parameters_get_as(self):
        params = ParamsDict({'key1': 'value1'})
        self.assertEqual(params.get_as('key1', str, 'other'), 'value1')

    def test_parameters_get_as_wrong_type(self):
        params = ParamsDict({'key1': 1})
        self.assertEqual(params.get_as('key1', str, 'other'), '1')

    def test_parameters_get_as_default(self):
        params = ParamsDict({'key1': 1})
        self.assertEqual(params.get_as('key2', str, 'other'), 'other')

    def test_parameters_get_list(self):
        params = ParamsDict({'key1': 'value1'})
        self.assertEqual(params.get_list('key1'), ['value1'])

    def test_get_error(self):
        params = ParamsDict({'key1': 'value1'})
        with self.assertRaises(KeyError):
            params['key2']

    def test_get_error2(self):
        params = ParamsDict({'key1': 'value1'})
        self.assertEqual(params['key1'], 'value1')

    def test_keys(self):
        params = ParamsDict({'key1': 'value1'})
        self.assertEqual(params.keys(), ['key1'])

    def test_get_list_of(self):
        params = ParamsDict({'key1': [1]})
        self.assertEqual(params.get_list_of('key1', str), ['1'])

    def test_get_list_of_not_found(self):
        params = ParamsDict({'key1': [1]})
        self.assertEqual(params.get_list_of('key2', str), [])



import functools
import logging
import re

from django.db import transaction, DatabaseError
from django.http import HttpResponse, StreamingHttpResponse, HttpRequest
from django.utils.datastructures import MultiValueDict

from savory_pie.context import APIContext
from savory_pie.django import validators
from savory_pie.errors import AuthorizationError, PreConditionError, MethodNotAllowedError
from savory_pie.formatters import JSONFormatter
from savory_pie.savory_newrelic import set_transaction_name
from savory_pie.helpers import get_sha1, process_get_request, process_post_request, process_put_request, process_delete_request

logger = logging.getLogger(__name__)


def batch_api_view(root_resource, base_regex):
    """
    View function factory that provides accessing to the resource tree
    rooted at root_resource.

    The produced function needs to be bound into URLs as r'^some/base/path/(.*)$'
    base_regex is the regex to the sub resources r'^some/base/path/(?P<base_resource>.*)$'
    """
    # Hide this import from sphinx
    from django.views.decorators.csrf import csrf_exempt

    if root_resource.resource_path is None:
        root_resource.resource_path = ''

    # TODO: Make this a setter
    root_resource.set_base_regex(base_regex)

    def create_request(method, uri, user):
        request = HttpRequest()
        request.path = uri
        request.method = method.upper()
        request.user = user

        return request

    def compute_resource_path(uri, host):
        url_path = host.join(uri.split(host)[1:])
        if url_path and url_path[0] == '/':
            url_path = url_path[1:]
        pattern = re.compile(root_resource.base_regex)
        match = pattern.search(url_path)
        if match:
            base_url = match.group('base_resource')
            if base_url:
                return base_url
        return ''

    def resource_dispatch(request, uri, data, host):

        resource_path = compute_resource_path(uri, host)
        ctx = compute_context(resource_path, request, root_resource)

        resource = ctx.resolve_resource_path(resource_path)

        resource_result = {'uri': uri}

        if resource is None:
            resource_result['status'] = 404
            return resource_result

        try:
            if request.method == 'GET':
                resource_result.update(
                    _get_request_for_batch(ctx, resource, data)
                )
            elif request.method == 'POST':
                resource_result.update(
                    _post_request_for_batch(ctx, resource, data)
                )
            elif request.method == 'PUT':
                resource_result.update(
                    _put_request_for_batch(ctx, resource, data)
                )
            elif request.method == 'DELETE':
                process_delete_request(ctx, resource)
                resource_result['status'] = 200
            else:
                raise MethodNotAllowedError(method=request.method)
        except MethodNotAllowedError:
            resource_result['status'] = 405
            resource_result['allowed'] = ','.join(resource.allowed_methods)
        except validators.ValidationError, ve:
            resource_result['status'] = 400
            resource_result['validation_errors'] = ve.errors
        except AuthorizationError as e:
            resource_result['status'] = 403
            resource_result['validation_errors'] = ['Modification of field {0} not authorized'.format(e.name)]
        except Exception:
            import traceback
            logger.exception('Caught Exception in API')
            resource_result['status'] = 500
            resource_result['data'] = {
                ctx.formatter.convert_to_public_property('error'): traceback.format_exc()
            }

        return resource_result

    def _get_request_for_batch(ctx, resource, data):
        resource_result = {}
        get_data = MultiValueDict()
        get_data.update(data)
        content_dict = process_get_request(ctx, resource, get_data)
        resource_result['status'] = 200
        resource_result['etag'] = get_sha1(ctx, content_dict)
        resource_result['data'] = content_dict

        return resource_result

    @_database_transaction_batch
    def _post_request_for_batch(ctx, resource, data):
        resource_result = {}
        new_resource = process_post_request(ctx, resource, data)
        resource_result['status'] = 201
        resource_result['location'] = ctx.build_resource_uri(new_resource)
        return resource_result

    @_database_transaction_batch
    def _put_request_for_batch(ctx, resource, data):

        resource_result = {}
        try:

            content_dict = process_put_request(ctx, resource, data)
        except PreConditionError:
            resource_result['status'] = 412
        except KeyError, ke:
            resource_result['status'] = 400
            resource_result['validation_errors'] = {'missingData': ke.message}
        else:
            if content_dict:
                resource_result['status'] = 200
                resource_result['data'] = content_dict
            else:
                resource_result['status'] = 204

        return resource_result

    @csrf_exempt
    @set_transaction_name
    def view(request, resource_path):

        ctx = compute_context(resource_path, request, root_resource)
        try:
            if resource_path or request.method != 'POST':
                return _not_allowed_resource_method(ctx, root_resource, request, ['POST'])

            data = ctx.formatter.read_from(request)
            result = []
            for resource_request in data.get('data', []):
                method = resource_request['method']
                uri = resource_request['uri']
                body = resource_request.get('body', None)

                resource_request = create_request(method, uri, request.user)

                result.append(
                    resource_dispatch(
                        resource_request,
                        uri,
                        body,
                        request.get_host()
                    )
                )

            return _content_success(ctx, None, request, {'data': result})

        except Exception:
            import traceback
            logger.exception('Caught Exception in API')
            return _internal_error(ctx, request, traceback.format_exc())

    return view


def compute_context(resource_path, request, root_resource):
    full_path = _strip_query_string(request.get_full_path())
    if len(resource_path) == 0:
        base_path = full_path
    else:
        base_path = full_path[:-len(resource_path)]

    ctx = APIContext(
        base_uri=re.sub('https?:', '', request.build_absolute_uri(base_path)),
        root_resource=root_resource,
        formatter=JSONFormatter(),
        request=request
    )

    return ctx


def api_view(root_resource):
    """
    View function factory that provides accessing to the resource tree
    rooted at root_resource.

    The produced function needs to be bound into URLs as r'^some/base/path/(.*)$'
    """
    # Hide this import from sphinx
    from django.views.decorators.csrf import csrf_exempt

    if root_resource.resource_path is None:
        root_resource.resource_path = ''

    @csrf_exempt
    @set_transaction_name
    def view(request, resource_path):

        ctx = compute_context(resource_path, request, root_resource)

        try:
            resource = ctx.resolve_resource_path(resource_path)

            if resource is None:
                return _not_found(ctx, request)

            if request.method == 'GET':
                return _process_get(ctx, resource, request)
            elif request.method == 'POST':
                return _process_post(ctx, resource, request)
            elif request.method == 'PUT':
                return _process_put(ctx, resource, request)
            elif request.method == 'DELETE':
                return _process_delete(ctx, resource, request)
            else:
                return _not_allowed_method(ctx, resource, request)
        except AuthorizationError as e:
            return _access_denied(ctx, field_name=e.name)
        except Exception:
            import traceback
            logger.exception('Caught Exception in API')
            return _internal_error(ctx, request, traceback.format_exc())

    return view


def _strip_query_string(path):
    return path.split('?', 1)[0]


class _RequestError(DatabaseError):
    """
    An error that will get raised when an endpoint returns anything other than 200 level response code.
    Since this is a subclass of `DatabaseError`, raising this error will cause the atomic block to rollback the changes.
    """
    pass


def _database_transaction_batch(func):
    @functools.wraps(func)
    def inner(ctx, resource, request, func=func):
        try:
            with transaction.atomic():
                response = func(ctx, resource, request)
                if not 200 <= response.get('status', 500) < 300:
                    # force a rollback
                    raise _RequestError
        except _RequestError:
            # The transaction should be rolled back
            # return the response
            return response

        return response

    def outer(ctx, resource, request):
        try:
            return inner(ctx, resource, request)
        except transaction.TransactionManagementError:
            return {'status': 409}
    return outer


def _database_transaction(func):
    @functools.wraps(func)
    def inner(ctx, resource, request, func=func):
        try:
            with transaction.atomic():
                response = func(ctx, resource, request)
                if not 200 <= response.status_code < 300:
                    # force a rollback
                    raise _RequestError
        except _RequestError:
            # The transaction should be rolled back
            # return the response
            return response

        return response

    def outer(ctx, resource, request):
        try:
            return inner(ctx, resource, request)
        except transaction.TransactionManagementError:
            return _transaction_conflict_to_response(ctx, resource, request)
    return outer


def _inner_transaction(ctx, resource, request, func):
    try:
        response = func(ctx, resource, request)
        if 200 <= response.status_code < 300:
            transaction.commit()
        else:
            transaction.rollback()
    except:
        transaction.rollback()
        raise
    return response


def _process_get(ctx, resource, request):
    try:
        content_dict = process_get_request(
            ctx,
            resource,
            request.GET
        )
        return _content_success(ctx, resource, request, content_dict)
    except MethodNotAllowedError:
        return _not_allowed_method(ctx, resource, request)


@_database_transaction
def _process_post(ctx, resource, request):
    try:
        data = ctx.formatter.read_from(request)
        new_resource = process_post_request(
            ctx,
            resource,
            data
        )
        return _created(ctx, request, request, new_resource)
    except validators.ValidationError, ve:
        return _validation_errors(ctx, ve.resource, request, ve.errors)
    except MethodNotAllowedError:
        return _not_allowed_method(ctx, resource, request)


@_database_transaction
def _process_put(ctx, resource, request):
    try:
        data = ctx.formatter.read_from(request)
        content_dict = process_put_request(
            ctx,
            resource,
            data,
            expected_hash=request.META.get('HTTP_IF_MATCH')
        )
        if content_dict:
            return _content_success(ctx, resource, request, content_dict)
        return _no_content_success(ctx, resource, request)
    except PreConditionError:
        return _precondition_failed(ctx, resource, request)
    except MethodNotAllowedError:
        return _not_allowed_method(ctx, resource, request)
    except validators.ValidationError, ve:
        return _validation_errors(ctx, resource, request, ve.errors)
    except KeyError, ke:
        return _validation_errors(ctx, resource, request, {'missingData': ke.message})


def _process_delete(ctx, resource, request):
    try:
        process_delete_request(ctx, resource)
        return _success(ctx, request, request)
    except MethodNotAllowedError:
        return _not_allowed_method(ctx, resource, request)


def _not_found(ctx, request):
    return HttpResponse(status=404)


def _access_denied(ctx, field_name=''):
    response = HttpResponse(status=403)
    ctx.formatter.write_to(
        {'validation_errors': ['Modification of field {0} not authorized'.format(field_name)]},
        response
    )
    return response


def _precondition_failed(ctx, resource, request):
    return HttpResponse(status=412)


def _not_allowed_method(ctx, resource, request):
    return _not_allowed_resource_method(ctx, resource, request, resource.allowed_methods)


def _not_allowed_resource_method(ctx, resource, request, methods):
    response = HttpResponse(status=405)
    response['Allowed'] = ','.join(resource.allowed_methods)
    return response


def _validation_errors(ctx, resource, request, errors):
    response = HttpResponse(status=400)
    ctx.formatter.write_to({'validation_errors': errors}, response)
    return response


def _created(ctx, resource, request, new_resource):
    response = HttpResponse(status=201)
    response['Location'] = ctx.build_resource_uri(new_resource)
    return response


def _content_success(ctx, resource, request, content_dict):
    if ctx.streaming_response:
        response = StreamingHttpResponse(
            content_dict,
            status=200,
            content_type=ctx.formatter.content_type)
        # No ETag, not practical on streaming
    else:
        response = HttpResponse(
            status=200,
            content_type=ctx.formatter.content_type
        )
        response['ETag'] = get_sha1(ctx, content_dict)
        ctx.formatter.write_to(content_dict, response)
    headers = ctx.headers
    if headers:
        for header, value in headers.items():
            response[header] = value

    return response


def _no_content_success(ctx, resource, request):
    return HttpResponse(status=204)


def _success(ctx, resource, request, content_dict=None):
    return HttpResponse(status=200)


def _internal_error(ctx, request, error):
    response = HttpResponse(status=500, content_type=ctx.formatter.content_type)
    error_body = {ctx.formatter.convert_to_public_property('error'): error}
    ctx.formatter.write_to(error_body, response)
    return response


def _transaction_conflict_to_response(ctx, resource, request):
    response = HttpResponse(status=409, content_type=ctx.formatter.content_type)
    error_body = _transaction_conflict(ctx, resource)
    ctx.formatter.write_to(error_body, response)
    return response


def _transaction_conflict(ctx, resource):
    return {ctx.formatter.convert_to_public_property('resource'): ctx.build_resource_uri(resource)}



import collections
import logging

import django.core.exceptions
from django.core.exceptions import ObjectDoesNotExist
from django.db.models.fields import FieldDoesNotExist
from django.utils.functional import Promise

from savory_pie import fields as base_fields
from savory_pie.errors import SavoryPieError

logger = logging.getLogger(__name__)


def average(query_list):
    # this will give you integer division
    return sum(query_list) / len(query_list)


AGGREGATE_MAPPER = {
    'sum': sum,
    'max': max,
    'min': min,
    'avg': average,
}


class DjangoField(base_fields.Field):
    def schema(self, ctx, **kwargs):
        model = kwargs['model']
        field_name = (model._meta.pk.name if self.name == 'pk' else self.name)
        # TODO: need to remove _field, field is a global state, we should not be changing it on an instance level
        self._field = None
        try:
            self._field = model._meta.get_field(field_name)
        except:
            # probably only for m2m fields
            try:
                self._field = model._meta.get_field_by_name(field_name)[0].field
            except FieldDoesNotExist:
                self._field = None

        schema = super(DjangoField, self).schema(ctx, **kwargs)
        if isinstance(self.validator, collections.Iterable):
            schema['validators'] = [validator.to_schema() for validator in self.validator]
        else:
            schema['validators'] = [self.validator.to_schema()]

        if self._field:
            _schema = {
                'blank': self._field.blank,
                'default': ctx.formatter.to_api_value(type(self._field.get_default()), self._field.get_default()),
                'helpText': self._field.help_text,
                'nullable': self._field.null,
                'readonly': not self._field.editable,
                'unique': self._field.unique
            }
            if self._field.choices:
                _schema['choices'] = self._field.choices
            if isinstance(_schema['helpText'], Promise):
                _schema['helpText'] = unicode(_schema['helpText'])
        else:
            _schema = {}

        return dict(schema.items() + _schema.items())


class AttributeField(base_fields.AttributeField, DjangoField):
    """
    Django extension of the basic AttributeField that adds support for optimized select_related
    or prefetch_related calls.

    Parameters:
            :class:`savory_pie.fields.AttributeField`

            ``use_prefetch``
                optional -- tells the attribute field to use
                prefetch_related rather than a select_related.  Defaults to false.

                There are two reasons you might need to do this...

                - select_related will not work when the foreign key allows null.
                - select_related will not work when the foreign key is a GenericForeignKey.

                See https://docs.djangoproject.com/en/dev/ref/models/querysets/

                This parameter is meaningless for top-level attributes.
    """
    def __init__(self, *args, **kwargs):
        self._use_prefetch = kwargs.pop('use_prefetch', False)
        super(AttributeField, self).__init__(*args, **kwargs)

    def prepare(self, ctx, related):
        related_attr = '__'.join(self._attrs[:-1])
        if related_attr:
            if self._use_prefetch:
                related.prefetch(related_attr)
            else:
                related.select(related_attr)

    def handle_incoming(self, ctx, source_dict, target_obj):
        super(AttributeField, self).handle_incoming(ctx, source_dict, target_obj)

    def save(self, target_obj):
        # TODO: remove this save call and track all models to save in the ctx.
        # Also run a topo-sort in the ctx and save models in the order.  We can
        # then remove all of the save order logic from the fields.
        target = self._get_object(target_obj)
        try:
            is_dirty = target.is_dirty
        except AttributeError:
            pass
        else:
            if not is_dirty():
                return
        target.save()

    def filter_by_item(self, ctx, filter_args, source_dict):
        filter_args[self._full_attribute] = source_dict.get(self._compute_property(ctx))

    def pre_save(self, model):
        return True


class AttributeFieldWithModel(AttributeField):
    """
    Django extension of the django AttributeField that adds support for setting up the multi-level expression
    with models, so it can be properly saved/added. This should only be used for multi-level expressioned attributes.

    Parameters:
            :class:`savory_pie.fields.AttributeField`

            ``model``
                A model class that is the model of the second last attribute, used to get at the object of this model

            ``extras``
                Extra values to be added to the filter call

    .. code-block:: python

        AttributeFieldWithModel('foo.property', type=int, model=ModelFoo, extras={'type': 'bar'})

    .. code-block:: javascript

        {'property': obj.foo}

    .. code-block:: python

        AttributeFieldWithModel('foo.bar.property', type=int, model=ModelBar)

    .. code-block:: javascript

        {'property': obj.foo.bar}

    """
    def __init__(self, *args, **kwargs):
        self._model = kwargs.pop('model', {})
        self._extras = kwargs.pop('extras', {})
        super(AttributeFieldWithModel, self).__init__(*args, **kwargs)

    def set_recursively(self, ctx, root_obj, attrs, source_dict):
        to_public = ctx.formatter.convert_to_public_property

        def get_model_params(target_type, ctx=ctx, source_dict=source_dict, extras=self._extras):
            # don't use key-values that do not apply to this target_type, or you'll
            # get an error from target_type.objects.get_or_create
            params = {}
            fields = map(lambda field: (field.name, field), target_type._meta.fields)
            for key, field in fields:
                if to_public(key) in source_dict:
                    params[key] = source_dict[to_public(key)]
                if key in extras:
                    params[key] = extras[key]
            return params, dict(fields)

        def get_or_create(target_type, attrs,
                          self=self, to_public=to_public, source_dict=source_dict, extras=self._extras):
            fields = target_type._meta.fields
            for field in filter(lambda f, attr=attrs[0]: f.name == attr, fields):
                # It might be necessary to get or create a prerequisite sub-object before
                # the object can be created.
                if isinstance(field, django.db.models.fields.related.ForeignKey) and not field.null:
                    extras[attrs[0]] = get_or_create(field.rel.to, attrs[1:])
            params, _ = get_model_params(target_type)
            obj, _ = target_type.objects.get_or_create(**params)
            return obj

        if len(attrs) == 1:
            params, fields = get_model_params(type(root_obj))
            for key in fields:
                if key in params:
                    setattr(root_obj, key, params[key])
            return root_obj
        try:
            # Descend to the next object down, easy if it exists
            if attrs[0] in [to_public(x.name) for x in type(root_obj)._meta.fields]:
                obj = getattr(root_obj, attrs[0])
                self.set_recursively(ctx, obj, attrs[1:], source_dict)
            else:
                obj = root_obj
            return obj
        except ObjectDoesNotExist:
            # Try to create the object.
            target_field_name, attrs = attrs[0], attrs[1:]
            target_type = getattr(type(root_obj), target_field_name).field.rel.to
            obj = get_or_create(target_type, attrs)
            setattr(root_obj, target_field_name, obj)
            if obj:
                return obj
            else:
                raise SavoryPieError(u'Unable to save attribute field: {0}'.format(self._full_attribute))

    def handle_incoming(self, ctx, source_dict, target_obj):
        obj = self.set_recursively(ctx, target_obj, self._attrs, source_dict)
        value = self.to_python_value(ctx, source_dict[self._compute_property(ctx)])
        setattr(obj, self._bare_attribute, value)

    def pre_save(self, model):
        return True


class URIResourceField(base_fields.URIResourceField, DjangoField):
    """
    Django extension of the basic URIResourceField that adds support for optimized
    select_related or prefetch_related calls.

    Parameters:
            :class:`savory_pie.fields.URIResourceField`

            ``use_prefetch``
                optional -- tells the attribute field to use
                prefetch_related rather than a select_related.  Defaults to false.

                There are two reasons you might need to do this...

                - select_related will not work when the foreign key allows null.
                - select_related will not work when the foreign key is a GenericForeignKey.

                See https://docs.djangoproject.com/en/dev/ref/models/querysets/

                This parameter is meaningless for top-level attributes.
    """
    def __init__(self, *args, **kwargs):
        self._use_prefetch = kwargs.pop('use_prefetch', False)
        super(URIResourceField, self).__init__(*args, **kwargs)

    def prepare(self, ctx, related):
        if self._use_prefetch:
            related.sub_prefetch(self._attribute)
        else:
            related.sub_select(self._attribute)

    def pre_save(self, model):
        return True


class URIListResourceField(base_fields.URIListResourceField, DjangoField):
    """
    Django extension of the basic URIListResourceField

        Parameters:
            :class:`savory_pie.fields.URIListResourceField`

            ``pre_save``
                optional -- tells the sub-model resource field whether to save
                before or after the related field.
    """
    def prepare(self, ctx, related):
        related.prefetch(self._attribute)

    def get_iterable(self, value):
        return value.all()

    def pre_save(self, model):
        return False


class SubModelResourceField(base_fields.SubObjectResourceField, DjangoField):
    """
    Django extension of the basic SubObjectResourceField that adds support for
    optimized select_related or prefetch_related calls.

        Parameters:
            :class:`savory_pie.fields.SubModelResourceField`

            ``use_prefetch``
                optional -- tells the sub-model resource field to use
                prefetch_related rather than a select_related.  Defaults to false.

                There are two reasons you might need to do this...

                - select_related will not work when the foreign key allows null.
                - select_related will not work when the foreign key is a GenericForeignKey.

                See https://docs.djangoproject.com/en/dev/ref/models/querysets/
    """
    def __init__(self, *args, **kwargs):
        self._use_prefetch = kwargs.pop('use_prefetch', False)
        self._skip_validation = kwargs.pop('skip_validation', False)
        super(SubModelResourceField, self).__init__(*args, **kwargs)

    def validate_resource(self, ctx, key, resource, source_dict):
        if self._skip_validation:
            return {}
        else:
            return super(SubModelResourceField, self).validate_resource(ctx, key, resource, source_dict)

    def prepare(self, ctx, related):
        if self._use_prefetch:
            related.prefetch(self._attribute)
            self._resource_class.prepare(ctx, related.sub_prefetch(self._attribute))
        else:
            related.select(self._attribute)
            self._resource_class.prepare(ctx, related.sub_select(self._attribute))

    def schema(self, ctx, **kwargs):
        kwargs = dict(kwargs.items() + {'schema': {'type': 'related', 'relatedType': 'to_one'}}.items())
        return super(SubModelResourceField, self).schema(ctx, **kwargs)

    def get_subresource(self, ctx, source_dict, target_obj):
        sub_source_dict = source_dict.get(self._compute_property(ctx)) or {}
        try:
            # Look at non-null FK
            sub_resource = super(SubModelResourceField, self).get_subresource(
                ctx,
                source_dict,
                target_obj
            )
        except django.core.exceptions.ObjectDoesNotExist:
            # Search by the source dict
            sub_resource = self._resource_class.get_by_source_dict(ctx, sub_source_dict)

        # Make sure the new model is attached
        if hasattr(sub_resource, 'model'):
            setattr(target_obj, self._attribute, sub_resource.model)
        return sub_resource

    def get_submodel(self, ctx, source_object):
        try:
            # Look at non-null FK
            sub_model = super(SubModelResourceField, self).get_submodel(
                ctx,
                source_object
            )
        except django.core.exceptions.ObjectDoesNotExist:
            sub_model = None

        return sub_model

    def _get_field(self, model):
        field_name = (model._meta.pk.name if self.name == 'pk' else self.name)
        field = None
        try:
            field = model._meta.get_field(field_name)
        except:
            # probably only for m2m fields
            try:
                field = model._meta.get_field_by_name(field_name)[0].field
            except FieldDoesNotExist:
                field = None
        return field

    def pre_save(self, model):
        '''
        This is to figure if we need to pre_save the foreign key or not.
        If Model A has foreign key to Model B, do everything normal
        If Model B has foreign key to Model A, you need to save Model A first before setting value on Model B
        @return: a Boolean variable used in ModelResources' put
        '''

        field = self._get_field(model)
        if field:
            attribute_name = field.related.field.name
            try:
                attribute = getattr(self._resource_class.model_class, attribute_name)
            except AttributeError:
                logger.debug('Setting pre_save to True with attribute %s', self._attribute)
                return True
            else:
                if isinstance(attribute, django.db.models.fields.related.ReverseSingleRelatedObjectDescriptor):
                    logger.debug('Setting pre_save to False with attribute %s and attribute_name %s',
                                 self._attribute, attribute_name)
                    return False

        return True


class OneToOneField(SubModelResourceField):
    """
    Django extension of the basic SubModelResourceField that is specifically for
    OneToOneField.

    When the resource can't be found, instead of looking it up with the source dictionary,
    it'll create a new instance for this resource since there is a 1-to-1 constraint.

    """

    def get_subresource(self, ctx, source_dict, target_obj):
        try:
            # Look at non-null FK
            sub_resource = super(SubModelResourceField, self).get_subresource(
                ctx,
                source_dict,
                target_obj
            )
        except django.core.exceptions.ObjectDoesNotExist:
            # create a new resource, since this is a one to one field
            sub_resource = self._resource_class.create_resource()

        # Make sure the new model is attached
        if hasattr(sub_resource, 'model'):
            setattr(target_obj, self._attribute, sub_resource.model)
        return sub_resource


class RelatedManagerField(base_fields.IterableField, DjangoField):
    """
    Django extension of the basic IterableField that adds support for
    optimized select_related or prefetch_related calls.

        Parameters:
            :class:`savory_pie.fields.IterableField`
    """
    def __init__(self, *args, **kwargs):
        super(RelatedManagerField, self).__init__(*args, **kwargs)

    def get_iterable(self, value):
        return sorted(value.all(), key=lambda x: x.pk)

    def prepare(self, ctx, related):
        attrs = self._attribute.replace('.', '__')
        related.prefetch(attrs)
        self._resource_class.prepare(ctx, related.sub_prefetch(attrs))

    def schema(self, ctx, **kwargs):
        dct = {'type': 'related', 'relatedType': 'to_many', 'fields': {}}
        if self._resource_class:
            for f in self._resource_class.fields:
                subkwargs = kwargs.copy()
                subkwargs['model'] = self._resource_class.model_class
                try:
                    dct['fields'][f._compute_property(ctx)] = f.schema(ctx, **subkwargs)
                except Exception, e:
                    logger.exception(e)    # field name is probably broken
        kwargs['schema'] = dct
        return super(RelatedManagerField, self).schema(ctx, **kwargs)

    def pre_save(self, model):
        '''
        This is to figure out if we need to pre_save the many to many field or not.
        In this case, we always want the related model to save first, before we save ourselves.
        @return: a Boolean variable used in ModelResources' put
        '''
        return False


class ReverseField(object):
    """
    Django field to handle the creation of new resources that require a foreign
    key to the parent in the API object graph.

    This field only runs on incoming requests on objects which have not been
    saved yet.

    Parameters:
        ``attribute_name``
            the name of the backref on the model.
    """
    def __init__(self, attribute_name):
        self.attribute_name = attribute_name

    @property
    def name(self):
        return self.attribute_name

    def handle_incoming(self, ctx, source_dict, target_obj):
        if target_obj.pk is not None:
            return
        setattr(target_obj, self.attribute_name, ctx.peek())

    def handle_outgoing(self, ctx, source_obj, target_dict):
        pass


class ReverseRelatedManagerField(object):
    """
    Where ReverseField works with one-to-one relationships, this works with
    many-to-one relationships where multiple A models all point to the same B.
    The PUT operation on AResource includes an embedded JSON chunk for a
    BResource, and when the PUT is executed, the BResource and its underlying
    B model are created. In this case, BResource.afield.handle_incoming needs
    to peek up the stack to get the A model, and set the B attribute on that
    A model to this B model.
    """
    from savory_pie.fields import ResourceClassUser
    __metaclass__ = ResourceClassUser

    def __init__(self, attribute_name, resource_class):
        self.attribute_name = attribute_name
        self.init_resource_class(resource_class)

    @property
    def name(self):
        return self.attribute_name

    def handle_outgoing(self, ctx, source_obj, target_dict):
        pass

    def pre_save(self, model):
        return True

    def handle_incoming(self, ctx, source_dict, target_obj):
        if target_obj.pk is not None:
            target = ctx.peek()
            a_model_type = self._resource_class.model_class
            assert type(target) == a_model_type
            setattr(target, self.attribute_name, target_obj)


class AggregateField(object):
    """
    Django field to handle aggregation objects. This uses
    something like aggregate(field) under the hood.

    Parameters:
        ''attribute''
            doted name of the field to count

        ''aggregate''
            django aggregation function

        ``published_property``
            optional -- name exposed in the API

    """
    def __init__(self, attribute, aggregate, published_property=None):
        self._attribute = attribute
        self._published_property = published_property
        self.aggregate = aggregate
        self._orm_attribute = self._attribute.replace('.', '__')

    def _compute_property(self, ctx):
        if self._published_property is not None:
            return ctx.formatter.convert_to_public_property(self._published_property)
        else:
            return ctx.formatter.convert_to_public_property(self._attribute)

    def _get(self, source_obj):
        try:
            return getattr(source_obj, '{0}__{1}'.format(self._orm_attribute, self.aggregate.name.lower()))
        except AttributeError:
            py_func = AGGREGATE_MAPPER.get(self.aggregate.name.lower(), None)
            if not py_func:
                raise

            query = source_obj.__class__.objects.filter(pk=source_obj.id).values(self._orm_attribute)
            # Since queries don't take into account None values we need to filter them out.
            # This is a potential bug further down the road if we do more intresting features like median
            # TODO: fix none values

            values = [item[self._orm_attribute] for item in query if item[self._orm_attribute] is not None]

            if not values:
                values.append(0)
            return py_func(values)

    def prepare(self, ctx, related):
        # Due to how annotate works with the django ORM, AggregateField can
        # only be used on a top level resource. We peek in to the stack to see
        # how deep we are and fail if it is too deep.
        try:
            ctx.peek(2)
        except IndexError:
            pass
        else:
            raise SavoryPieError(
                'RelatedCountField can only be used on a top level ModelResource'
            )
        related.annotate(
            self.aggregate,
            self._orm_attribute,
            distinct=True,
        )

    def handle_incoming(self, ctx, source_dict, target_obj):
        pass

    def handle_outgoing(self, ctx, source_obj, target_dict):
        target_dict[self._compute_property(ctx)] = ctx.formatter.to_api_value(
            int,
            self._get(source_obj)
        )


class RelatedCountField(AggregateField):
    """
    Django field to handle counting the number of related objects. This uses
    something like annotate(Count()) under the hood.

    Parameters:
        ''attribute''
            doted name of the field to count

        ``published_property``
            optional -- name exposed in the API
    """
    def __init__(self, attribute, published_property=None):
        super(RelatedCountField, self).__init__(attribute, django.db.models.Count, published_property)

    def _get(self, source_obj):
        try:
            return getattr(source_obj, '{0}__{1}'.format(self._orm_attribute, self.aggregate.name.lower()))
        except AttributeError:
            return source_obj.__class__.objects.filter(pk=source_obj.id).values(self._orm_attribute).count()



import unittest
from mock import Mock
from savory_pie.django.auth import DjangoUserPermissionValidator


class DjangoUserPermissionValidatorTestCase(unittest.TestCase):

    def test_target_source_changed(self):
        validator = DjangoUserPermissionValidator('value')
        ctx = Mock(spec=['user'])
        ctx.request = Mock()
        ctx.request.user.has_perm.return_value = False
        self.assertFalse(validator.is_write_authorized(ctx, None, 'a', 'b'))
        ctx.request.user.has_perm.assert_called_with('value')

    def test_target_source_not_changed(self):
        validator = DjangoUserPermissionValidator('value')
        ctx = Mock(spec=['user'])
        ctx.request = Mock()
        # Should not call has_perm
        ctx.request.user.has_perm.side_effect = Exception
        self.assertTrue(validator.is_write_authorized(ctx, None, 'a', 'a'))
